@@ -16,13 +16,11 @@ from uuid import uuid4
 import boto3
 import pytest
 
-from object_clerk import ArchiveClerk
 from object_clerk import ObjectClerk
 from object_clerk import ObjectNotFoundException
 from object_clerk import ObjectSaveException
 from object_clerk.config import MULTIPART_THRESHOLD
 
-ARCHIVE_STORE_USE_SSL = environ.get("ARCHIVE_STORE_USE_SSL", "False").lower() == "true"
 
 """
 S3 endpoint configuration
@@ -49,7 +47,7 @@ def get_abs_path(test_filename: str = None) -> str:
     :return: absolute path of the test file
     """
     pwd = os.getcwd()
-    return os.path.join(pwd, test_filename)
+    return os.path.join(pwd, "object_clerk", "tests", test_filename)
 
 
 """
@@ -63,7 +61,7 @@ def boto_client() -> boto3.client:
     Configure a boto3.client for s3
     :return: boto3.client instance
     """
-    schema = "https" if ARCHIVE_STORE_USE_SSL or PORT == "443" else "http"
+    schema = "http"
     endpoint_url = f"{schema}://{HOST}:{PORT}/"
     return boto3.client(
         service_name="s3",
@@ -167,7 +165,20 @@ def object_store_data(boto_client, bucket) -> None:
     boto_client.delete_object(Bucket=bucket, Key="move_file")
 
 
-@pytest.fixture(scope="function", params=[1, 3, 5, 8, 13, 16, 17, 30, 50])
+@pytest.fixture(
+    scope="function",
+    params=[
+        1,
+        3,
+        5,
+        8,
+        pytest.param(13, marks=pytest.mark.long),
+        pytest.param(16, marks=pytest.mark.long),
+        pytest.param(17, marks=pytest.mark.long),
+        pytest.param(30, marks=pytest.mark.long),
+        pytest.param(50, marks=pytest.mark.long),
+    ],
+)
 def local_file(tmp_path_factory, request) -> Tuple[Path, int]:
     """
     Creates files locally of varying sizes based upon fixture params
@@ -215,23 +226,13 @@ def object_clerk() -> ObjectClerk:
     Fixture providing a configured instance of the Object clerk
     :return: ObjectClerk instance
     """
-    return ObjectClerk(host=HOST, port=PORT, access_key=ACCESS_KEY, secret_key=SECRET_KEY, **RETRY)
-
-
-# stub for future testing
-@pytest.fixture(scope="session")
-def archive_clerk() -> ArchiveClerk:
-    """
-    Fixture providing a configured instance of the Archive clerk
-    :return: ArchiveClerk instance
-    """
-    return ArchiveClerk(
+    return ObjectClerk(
         host=HOST,
         port=PORT,
         access_key=ACCESS_KEY,
         secret_key=SECRET_KEY,
-        use_ssl=ARCHIVE_STORE_USE_SSL,
         **RETRY,
+        list_objects_page_size=1,  # force handling of multiple calls to object store
     )
 
 
@@ -546,45 +547,26 @@ def test_list_objects_not_truncated(object_clerk, bucket, object_store_data):
     :param object_store_data: Fixture that loaded objects
     :return: None
     """
-    contents, is_truncated = object_clerk.list_objects(bucket)
-
-    # because object_clerk is session scoped, can be from 2 to 4 objects, depending on the order that tests are run
-    assert len(contents) >= 2
-    assert not is_truncated
+    contents = object_clerk.list_objects(bucket)
+    # because object_clerk is session scoped, can be from 2 to 4 objects,
+    # depending on the order that tests are run
     object_ids = [content["Key"] for content in contents]
+    assert len(object_ids) >= 2
     assert "copy_file" in object_ids
     assert "get_file" in object_ids
 
 
-@pytest.mark.development
-def test_list_objects_truncated(object_clerk, bucket, object_store_data):
-    """
-    Given: ObjectClerk instance, objects that exist in bucket
-    When: Querying what objects exist in a bucket
-    Then: List of bucket contents is obtained
-    :param object_clerk: ObjectClerk instance
-    :param bucket: Bucket the existing object is in
-    :param object_store_data: Fixture that loaded objects
-    :return: None
-    """
-    contents, is_truncated = object_clerk.list_objects(bucket, 1)
-
-    assert len(contents) == 1
-    assert is_truncated
-
-
 @pytest.mark.development
 def test_list_objects_invalid_bucket(object_clerk, bucket, object_store_data):
     with pytest.raises(ObjectNotFoundException):
-        object_clerk.list_objects("false_bucket")
+        next(object_clerk.list_objects("false_bucket"))
 
 
 @pytest.mark.development
 def test_list_objects_empty_bucket(object_clerk, empty_bucket):
-    contents, is_truncated = object_clerk.list_objects(empty_bucket, 1)
-
-    assert contents == []
-    assert not is_truncated
+    contents = object_clerk.list_objects(empty_bucket)
+    with pytest.raises(StopIteration):
+        next(contents)
 
 
 @pytest.mark.development
@@ -598,11 +580,10 @@ def test_list_objects_prefix(object_clerk, bucket, object_store_data):
     :param object_store_data: Fixture that loaded objects
     :return: None
     """
-    contents, is_truncated = object_clerk.list_objects(bucket, prefix="cop")
-
+    contents = object_clerk.list_objects(bucket, prefix="cop")
+    contents = list(contents)
     assert len(contents) == 1
     assert contents[0]["Key"] == "copy_file"
-    assert not is_truncated
 
 
 def test_pass():
